'use strict';

const AWS = require('aws-sdk')
const dynamodb = new AWS.DynamoDB();

// Environment variable for DynamoDB table
const tableName = process.env.TABLE_NAME;
// Environment variable for AWS Organizations service endpoint
const organizationsEndpoint = process.env.ORGANIZATIONS_ENDPOINT;

// Not all the regions have Organizations service endpoints
const splitArray = organizationsEndpoint.split(".");
const ep = new AWS.Endpoint(organizationsEndpoint);
const organizations = new AWS.Organizations({endpoint:ep,region:splitArray[1]});


// Call the ListAccounts API to retrieve the status for all the linked accounts
function refreshAccounts() {
	console.log("Enter refreshAccounts");

	var newData = {Accounts:[]};
	
	var handleError = function(response) {
		console.log(response.err, response.err.stack); // an error occurred
		throw response.err;
	};
	
	var params = {};
	organizations.listAccounts(params).on('success', function handlePage(response) {
		console.log("ListAccounts response: \n" + JSON.stringify(response.data));    
		var account;
			
		// extract each account's Id and Status 
		for (account of response.data.Accounts) {
			var accountStatus = {Id: account.Id, Status: account.Status};
			newData.Accounts.push(accountStatus);
		}
	
		// handle response pagination
		if (response.hasNextPage()) {
			response.nextPage().on('success', handlePage).on('error', handleError).send();
		} else {
			console.log("Account Status: \n" + JSON.stringify(newData)); 
			saveDataToDynamo(JSON.stringify(newData));
		}
	}).on('error', handleError).send();
}

// Save the account status JSON to the DynamoDB table if it is different than the existing data in the table
function saveDataToDynamo(data) {
	console.log("enter saveDataToDynamo");

	var params = {
		ExpressionAttributeNames: {
			"#ACC": "Accounts"
		}, 
		ExpressionAttributeValues: {
			":t": {
				S: data
			} 
		}, 
		Key: {
			"Organization": {
				S: "default"
			}
		}, 
		ConditionExpression: "#ACC <> :t",
		ReturnValues: "ALL_NEW", 
		TableName: tableName, 
		UpdateExpression: "SET #ACC = :t"
	};
	
	dynamodb.updateItem(params, function(err, data) {
		console.log("DynamoDB callback...");  
		if (err) {
			console.log(err, err.stack); // an error occurred
		
			if (err.code != "ConditionalCheckFailedException") {
				throw err;
			}
		} else {
			console.log(data);           // successful response
		}
	});
}

exports.handler = (event, context) => {
	console.log('entry handler');
	refreshAccounts();

};
